//+------------------------------------------------------------------+
//|                                                 RangeBreakEA.mq4 |
//|                                 Copyright 2014, TradeAndSoftware |
//|                                                 http://tasfx.net |
//+------------------------------------------------------------------+
#property copyright "Copyright 2014, TradeAndSoftware"
#property link      "http://tasfx.net"
#property version   "1.00"
#property strict

extern int Magic = 20150211;
extern string comment = "RangeBreakEA";
extern double Lots = 0.1;
extern double MaxSpread = 5.0;
extern int Slippage = 3;
extern int StopLoss = 45;
extern int TakeProfit = 80;

extern int GMTOffset = 3;
extern double GMTOpenHour = 10.0;
extern double GMTCloseHour = 18.0;
extern double GMTAllCloseHour = 25.0;

extern int RangePeriod = 8;
extern int RangeWidth = 80;

double OpenHour;
double CloseHour;
double AllCloseHour;




//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   double tp = 0.0;
   double sl = 0.0;

   if(CountOrders() > 0)
   {
      //TrailingStop(StopLoss);
   }


   if(CountOrders() == 0 && IsTradePossible())
   {
      double high = High[iHighest(NULL, 0, MODE_HIGH, RangePeriod, 2)];
      double low = Low[iLowest(NULL, 0, MODE_LOW, RangePeriod, 2)];
      double diff = high - low;
      double range = RangeWidth * Point;
      
      int ticket = 0;
      if(range > diff)
      {
         if(Close[1] > high)
         {
            //buy
            sl = Ask - (StopLoss * Point);
            //sl = low;
            tp = Ask + TakeProfit * Point;
            ticket = OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,sl,tp,comment,Magic,0,Blue);
         }
         else if(Close[1] < low)
         {
               //sell
               sl = Bid + (StopLoss * Point);
               //sl = high;
               tp = Bid - TakeProfit * Point;
               ticket = OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,sl,tp,comment,Magic,0,Red);
         }
      }
   }
      
   
  }
//+------------------------------------------------------------------+

int CountOrders()
{
   int count = 0;
   for(int i=0; i<=OrdersTotal()-1; i++)
   {
       if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES) == false) break;
       if(OrderMagicNumber() != Magic || OrderSymbol() !=Symbol()) continue;
       count++;
   }
   return(count);   
}

bool IsTradePossible()
{
   bool IsTradePossible = true;
	double nowHour = Hour() + Minute() / 60.0;
	SummerTimer();
	
	//Trade Possible Time
	if(OpenHour < CloseHour) 
	{
		if(nowHour < OpenHour || nowHour >= CloseHour)
		{
			IsTradePossible = false;
	   }
	} 
	else if(nowHour < OpenHour && nowHour >= CloseHour)
	{		
	   IsTradePossible = false;
	}
	else if(TimeDayOfWeek(TimeCurrent()-GMTOffset*3600) == 5) //Friday not trade
	{
		IsTradePossible = false;
	}
   return(IsTradePossible);
}

void SummerTimer()
{
   bool isSummerTime = false;
	if(Month() > 3 && Month() < 11)
	{
	   isSummerTime = true;
	}
	else if(Month() == 3 && Day() > 7 + DayOfWeek())
	{
		isSummerTime = true;
   }
   else if(Month() == 11 && Day() > DayOfWeek())
   {
      isSummerTime = true;
   }
   
   if(isSummerTime)
   {
   	OpenHour = GMTOpenHour + GMTOffset - 1.0;
   	CloseHour = GMTCloseHour + GMTOffset - 1.0;
   	AllCloseHour = GMTAllCloseHour + GMTOffset - 1.0;
   }
   else
   {
   	OpenHour = GMTOpenHour + GMTOffset;
   	CloseHour = GMTCloseHour + GMTOffset;
   	AllCloseHour = GMTAllCloseHour + GMTOffset;
   }   
   SetTiming();
}

void SetTiming()
{
	if(OpenHour < 0.0)
	{
		OpenHour+=24.0;
   }
	else if(OpenHour >= 24.0)
	{
		OpenHour-=24.0;
   }
   
	if(CloseHour < 0.0)
	{
		CloseHour+=24.0;
   }
	else if(CloseHour >= 24.0)
	{
		CloseHour-=24.0;
   }
   
   if(AllCloseHour < 0.0)
   {
      AllCloseHour+=24.0;
   }
   else if(AllCloseHour >= 24.0)
   {
      AllCloseHour-=24;
   }
   
}  


bool IsAllCloseTime()
{
   bool isCloseTime = false;
	double nowHour = Hour() + Minute() / 60.0;
	
	if(!IsTradePossible() && AllCloseHour < nowHour) 
	{
      isCloseTime = true;
	} 
   return(isCloseTime);
}

void CloseOrder()
{
   bool isExitBuy = false;
   bool isExitSell = false;

   if(IsAllCloseTime())
   {
      isExitBuy = true;
      isExitSell = true;
   }
   
	if(isExitBuy || isExitSell) 
	{ 
		for(int i=0;i<CountOrders();i++) 
		{
			if(OrderSelect(i,SELECT_BY_POS, MODE_TRADES))
			{
				if(OrderSymbol()==Symbol() && OrderMagicNumber()==Magic)
				{
				   if((OrderType() == OP_BUY && isExitBuy) || (OrderType() == OP_SELL && isExitSell))
				   {
					   CloseAll();
					} 
			   }
		   }
		}
	} 
}
  
void CloseAll() 
{
	for (int i=CountOrders()-1;i>=0;i--) 
	{
		if (OrderSelect(i,SELECT_BY_POS))
		{
   		if (OrderSymbol()==Symbol() && OrderMagicNumber()==Magic)
   		{
      	   if(Ask-Bid<=MaxSpread)
      	   {
               if(OrderType() == OP_BUY)
               {
      				if(!OrderClose(OrderTicket(),OrderLots(),Bid,Slippage,Navy)) {
     					   //Error
     					}
     				}
     				else if(OrderType() == OP_SELL)
     				{
      				if(!OrderClose(OrderTicket(),OrderLots(),Ask,Slippage,Maroon)) {
     					   //Error
      				}
      			}
            }
            else
		      {
      		   string spread = DoubleToStr(GetSpread(), 1);
      		   Print("Spread Over!(Close) : " + Symbol() + " Spead=" + spread);
      		}      
   	   }
   	}
	}
}  

double GetSpread()
{
   int digit = Digits;
   if(digit == 3 || digit == 5)
   {
      digit -= 1;
   }
   double spreadTemp = (Ask - Bid) * MathPow(10,digit);
   return(spreadTemp);
}

void TrailingStop(int ts)
{
   double newSL = 0.0;
   for(int i=0; i<OrdersTotal(); i++)
   {
      if(OrderSelect(i, SELECT_BY_POS) == false) break;
      if(OrderSymbol() != Symbol() || OrderMagicNumber() != Magic) continue;
      if(OrderType() == OP_BUY)
      {
         newSL = Bid - ts * Point;
         if(newSL >= OrderOpenPrice() && newSL > OrderStopLoss())
         {
            OrderModify(OrderTicket(), 0, newSL, 0, 0);
            break;
         }
      }
      else if(OrderType() == OP_SELL)
      {
         newSL = Ask + ts * Point;
         if(newSL <= OrderOpenPrice() && (newSL < OrderStopLoss() || OrderStopLoss() == 0))
         {
            OrderModify(OrderTicket(), 0, newSL, 0, 0);
            break;
         }
      }
   }

}